﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Mvc.Ajax;
using System.Security;
using System.Security.Principal;

using EcoMvcExample.Model;
using EcoMvcExample.MvcWebsite.Models;
using Adecs.Eco.Helpers;

using Eco.Handles;
using Eco.ObjectRepresentation;
using Eco.Services;


namespace EcoMvcExample.MvcWebsite.Controllers
{
    public class TasksController : EcoMvcExampleBaseController
    {
        public ActionResult Index()
        {
            if (Request.IsAuthenticated)
            {
                if (User.Identity != null)
                {
                    IIdentity ident = User.Identity;
                    string sUsername = ident.Name;

                    // Get this users tasks.  This is where you would use your business model to
                    // retrieve your data.  In this case we are using ECO to obtain our objects.
                    //
                    // In this instance I've used a static class and method to be responsible for
                    // the retrieval of the objects, so that it potentially could be tested separately
                    // and allow more easy reuse elsewhere in the future.
                    IList<Task> tasks = TaskHelper.GetUserTasks(this.EcoSpace, sUsername);

                    ViewData["Title"] = "My Tasks";
                    CheckIfUserCanCreateTasks();

                    return View(tasks);
                }
                else
                {
                    throw new UnauthorizedAccessException();
                }
            }
            else
            {
                throw new UnauthorizedAccessException();
            }
        }

        public ActionResult All()
        {
            // Add action logic here
            if (Request.IsAuthenticated)
            {
                // Again, get this users tasks.  This is where you would use your business model
                // to retrieve your data.  In this case we are using ECO to obtain our objects.
                IList<Task> allTasks = TaskHelper.GetAllTasks(this.EcoSpace);

                ViewData["Title"] = "All Tasks";
                CheckIfUserCanCreateTasks();

                return View(allTasks);
            }
            else
            {
                throw new UnauthorizedAccessException();
            }
        }

        public ActionResult Details(string id)
        {
            if (Request.IsAuthenticated)
            {
                // Get the ECO Object relating to the given ID
                IExternalIdService es = EcoSpace.GetEcoService<IExternalIdService>();
                IObjectInstance oi = es.ObjectForId(id);

                Task objInstance = oi.AsObject as Task;
                ViewData["Title"] = "View Task";
                CheckIfUserCanCreateTasks();

                return View(objInstance);
            }
            else
            {
                throw new UnauthorizedAccessException();
            }
        }

        public ActionResult Create()
        {
            if (Request.IsAuthenticated)
            {
                if (User.Identity != null)
                {
                    IIdentity ident = User.Identity;
                    string sUsername = ident.Name;

                    // Grab this user's record.
                    TaskUser thisUser = TaskUserHelper.GetUser(this.EcoSpace, sUsername);
                    if (thisUser.CanCreateTasks)
                    {
                        // Get the Object representing the user being assigned this Task
                        ExpressionHandle users = TaskUserHelper.GetAllUsers(this.EcoSpace);
                        SelectList si = new SelectList(users, "ExternalId", "Username", EcoHelper.IdForObject(thisUser.AsIObject()));

                        ViewData["Title"] = "Create New Task";
                        CheckIfUserCanCreateTasks();

                        return View(si);
                    }
                    else
                    {
                        throw new UnauthorizedAccessException();
                    }
                }
                else
                {
                    throw new UnauthorizedAccessException();
                }
            }
            else
            {
                throw new UnauthorizedAccessException();
            }
        }

        /// This instance of Create takes the post back from the previous form, and does the
        /// actual grunt work of creating the new Task.
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Create(string taskuser, string taskname, string tasknotes)
        {
            if (string.IsNullOrEmpty(taskuser))
            {
                ModelState.AddModelError("taskuser", "You must select a user for this Task.");
            }
            if (string.IsNullOrEmpty(taskname))
            {
                ModelState.AddModelError("taskname", "You must specify a new Name for this Task.");
            }


            if (Request.IsAuthenticated)
            {
                if (User.Identity != null)
                {
                    IIdentity ident = User.Identity;
                    string sUsername = ident.Name;

                    // Grab this user's record.
                    TaskUser thisUser = TaskUserHelper.GetUser(this.EcoSpace, sUsername);

                    if (thisUser.CanCreateTasks)
                    {
                        if (ViewData.ModelState.IsValid)
                        {
                            // If we are here, then we have all the necessary information we require to
                            // create the ECO Object.
                            IObjectInstance oiUser = EcoHelper.ObjectFromId(taskuser, this.EcoSpace);
                            TaskUser tuOwner = oiUser.AsObject as TaskUser;

                            if (tuOwner != null)
                            {
                                Task newTask = new Task(this.EcoSpace);
                                newTask.User = tuOwner;
                                newTask.Name = taskname;
                                newTask.DateEntered = DateTime.Now;
                                newTask.Completed = false;
                                newTask.Notes = tasknotes;

                                // Commit the object back to the database.
                                EcoMvcExampleEcoSpace es = this.EcoSpace as EcoMvcExampleEcoSpace;
                                if (es != null)
                                    es.UpdateDatabase();

                                return RedirectToAction("Index", "Tasks");
                            }
                            else
                            {
                                throw new ArgumentNullException("tuOwner");
                            }
                        }
                        else
                        {
                            // If we are here, the model had errors, and this should redisplay the form
                            // with validation summary errors, and fields highlighted that require a value.
                            ExpressionHandle users = TaskUserHelper.GetAllUsers(this.EcoSpace);
                            SelectList si = new SelectList(users, "ExternalId", "Username", EcoHelper.IdForObject(thisUser.AsIObject()));

                            ViewData["Title"] = "Create New Task";
                            CheckIfUserCanCreateTasks();

                            return View(si);
                        }
                    }
                    else
                    {
                        throw new UnauthorizedAccessException();
                    }
                }
                else
                {
                    throw new UnauthorizedAccessException();
                }
            }
            else
            {
                throw new UnauthorizedAccessException();
            }
        }
    }
}
