using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Web;
using System.Web.Services;
using Eco.Handles;
using Eco.ObjectRepresentation;
using Eco.Services;
using Eco.Web;

using EcoMvcExample.Model;


namespace EcoMvcExample.MvcWebsite
{
	/// <summary>
	/// Summary description for $WebServiceClassName$.
	/// </summary>
	[WebService(Namespace = "http://tempuri.org/")]
	[WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
	[ToolboxItem(false)]
	public class EcoWebServiceExample : System.Web.Services.WebService
	{
		private EcoMvcExample.EcoMvcExampleEcoSpace m_EcoSpace;
		private EcoSpaceStrategyHandler GetStrategyHandler()
		{
			return new EcoSpaceStrategyHandler(EcoSpaceStrategyHandler.SessionStateMode.Never, typeof(EcoMvcExample.EcoMvcExampleEcoSpace) );
		}
		
		private EcoMvcExample.EcoMvcExampleEcoSpace EcoSpace
		{
			get
			{
				if (m_EcoSpace == null)
				{
					m_EcoSpace = GetStrategyHandler().GetSessionFreeEcoSpace() as EcoMvcExample.EcoMvcExampleEcoSpace;
				}
				return m_EcoSpace;
			}
		}

		/// <summary>
		/// Call DoneWithEcoSpace at end of each Web service to return EcoSpace to pool
		/// </summary>
		private void DoneWithEcoSpace()
		{
			if (m_EcoSpace != null) 
				GetStrategyHandler().ReturnSessionFreeEcoSpace(m_EcoSpace);
		}

		#region Utility methods
		protected void UpdateDatabase()
		{
			IPersistenceService persistenceService = EcoSpace.GetEcoService<IPersistenceService>();
			IDirtyListService dirtyListService = EcoSpace.GetEcoService<IDirtyListService>();
			persistenceService.UpdateDatabaseWithList(dirtyListService.AllDirtyObjects());
		}

		protected string IdForObject(IObject obj)
		{
			return EcoSpace.GetEcoService<IExternalIdService>().IdForObject(obj);
		}

		protected IObjectInstance ObjectForId(string id)
		{
			return EcoSpace.GetEcoService<IExternalIdService>().ObjectForId(id);
		}
		#endregion

		// Sample Web Service Methods
		// The following methods are provided to allow for testing a new web service.
		// To test, uncomment these lines, save, and build the project.
		// This code may safely be deleted.
        //
        // These methods are created by default when you create a new Eco Web Service.
        // They are showing the default example code, which is not related to my examples model.
		/*
		[WebMethod]
		public string HelloWorld()
		{
			return "Hello World";
		}

		// This is a minimal web method reading data
		// It assumes an ECO model containing a Class named Order
		[WebMethod]
		int OrderCount()
		{
			IOclService oclService = EcoSpace.GetEcoService<IOclService>();
			IElement resultElement = oclService.EvaluateAndSubscribe("Order.allInstances->size");
			int result = resultElement.GetValue<int>();
			DoneWithEcoSpace();
			return result;
		}

		// This is a minimal web method modifying data
		// It assumes an ECO model containing a Class named Order
		// with a string attribute Order.CustomerName
		[WebMethod]
		void NewOrder(string name)
		{
			Order anOrder = new Order(EcoSpace);
			anOrder.CustomerName = name;
			UpdateDatabase();
			DoneWithEcoSpace();
		}
		*/

        [WebMethod]
        // Here we are going to take the name of the specific person, and return their tasks
        public Task[] GetUsersOutstandingTasks(string sUsersName)
        {
            try
            {
                return null;
            }
            finally
            {
                // Need to ensure we call DoneWithEcoSpace once we are finished, so I wrapped this 
                // functionality in a try..finally block to ensure it's cared for.
                DoneWithEcoSpace();
            }
        }
	}
}
