﻿using System;
using System.Collections.Generic;

using Eco.ObjectRepresentation;
using Eco.Services;

using EcoMvcExample.Model;
using Adecs.Eco.Helpers;


namespace EcoMvcExample.MvcWebsite.Models
{
    public static class TaskHelper
    {
        public static IList<Task> GetUserTasks(IEcoServiceProvider ecoProvider, string UserName)
        {
            if (ecoProvider == null)
                throw new ArgumentNullException("ecoProvider");
            if (UserName == null)
                throw new ArgumentNullException("UserName");

            // Get the specified user
            IVariableFactoryService vfs = ecoProvider.GetEcoService<IVariableFactoryService>();
            IModifiableVariableList mvl = vfs.CreateVariableList();
            mvl.AddConstant("vUserName", UserName);

            // Return a list of the Tasks by specific user
            GeneralObjectRetrieval gor = new GeneralObjectRetrieval(ecoProvider);
            IList<TaskUser> users = gor.GetObjectsMatchingCriteria<TaskUser>("->select(tu | tu.Username = vUserName)", mvl);
            if (users == null)
            {
                throw new ArgumentNullException("users");
            }
            else
            {
                if (users.Count > 0)
                {
                    TaskUser user = users[0];

                    // Select the users tasks in decending order
                    IModifiableVariableList uservl = vfs.CreateVariableList();
                    uservl.AddConstant("vUser", user);

                    IList<Task> tasks = gor.GetObjectsMatchingCriteria<Task>("->select(t | t.User = vUser)->orderDescending(DateEntered)", uservl);
                    return tasks;
                }
                else
                {
                    return null;
                }
            }
        }

        public static IList<Task> GetAllTasks(IEcoServiceProvider ecoProvider)
        {
            if (ecoProvider == null)
                throw new ArgumentNullException("ecoProvider");

            GeneralObjectRetrieval gor = new GeneralObjectRetrieval(ecoProvider);
            return gor.GetObjects<Task>();
        }

        /* This is currently commented out until it is correctly identified what is needed.
        public static Task GetTask(IEcoServiceProvider ecoProvider)
        {
            if (ecoProvider == null)
                throw new ArgumentNullException("ecoProvider");
            if (BrandName == null)
                throw new ArgumentNullException("BrandName");

            IVariableFactoryService vfs = ecoProvider.GetEcoService<IVariableFactoryService>();
            IModifiableVariableList mvl = vfs.CreateVariableList();
            mvl.AddConstant("vBrandName", BrandName);

            GeneralObjectRetrieval gor = new GeneralObjectRetrieval(ecoProvider);
            IList<Brand> brands = gor.GetObjectsMatchingCriteria<Brand>("->select(b | b.Name = vBrandName)", mvl);
            if (brands == null)
            {
                throw new ArgumentNullException("brands");
            }
            else
            {
                if (brands.Count > 0)
                {
                    return brands[0];
                }
                else
                {
                    return null;
                }
            }
        } */
    }
}
